@@ -1,8 +1,11 @@
+import os
+import struct
 import sys
 import re
 import time
 from enum import Enum
-from multiprocessing import Queue
+
+IPC_FIFO_NAME = "mpscreen_fifo"
 
 
 class Align(Enum):
@@ -116,31 +119,40 @@ class Vint(_Val):
 
 # CLIENTS
 
+def encode_msg_size(size: int) -> bytes:
+    return struct.pack("<I", size)
+
+
+def create_msg(content: bytes) -> bytes:
+    size = len(content)
+    return encode_msg_size(size) + content
+
+
+def put(msg: bytes):
+    to_write = create_msg(msg)
+    fifo = os.open(IPC_FIFO_NAME, os.O_WRONLY)
+    os.write(fifo, to_write)
+    os.close(fifo)
+
+
 class line:
-    def __init__(self, q: Queue, id, attributes):
-        self.q = q
+    def __init__(self,  id, attributes, translation):
         self.id = id
         self.attributes = attributes
-        self.lastUpload = 0
-        self.addBuffer = {}
+        self.translation = translation
 
     def __setattr__(self, key, value):
-        if key in ["q", "id", "attributes", "lastUpload","addBuffer"]:
+        if key in ["q", "id", "attributes","translation"]:
             self.__dict__[key] = value
         else:
             if key in self.attributes:
-                self.q.put([self.id, key, False, value])
+                self._set(value, key)
 
     def _add(self, value, item):
-        t = time.time()
-        if t - self.lastUpload < 0.1:
-            self.addBuffer[item] = [True, value + 0 if item not in self.addBuffer else self.addBuffer[item]]
-        else:
-            self.q.put([self.id, item, True, value + 0 if item not in self.addBuffer else self.addBuffer[item]])
-
+        put(bytes([self.id, int(self.translation[item]), 1]) + int(value).to_bytes(4, byteorder='little'))
 
     def _set(self, value, item):
-        self.q.put([self.id, item, False, value])
+        put(bytes([self.id, int(self.translation[item]), 0]) + str(value).encode("utf-8"))
 
     def __setitem__(self, item, val):
         self.__setattr__(item, val)
@@ -154,15 +166,14 @@ class line:
 
 
 class buffer:
-    def __init__(self, q: Queue, id):
-        self.q = q
+    def __init__(self, id):
         self.id = id
 
     def append(self, message):
-        self.q.put([self.id, True, message])
+        put(bytes([self.id, 1]) + message.encode("utf-8"))
 
-    def clear(self, message):
-        self.q.put([self.id, False, message])
+    def clear(self):
+        put(bytes([self.id, 0]))
 
 
 # SERVERS
@@ -206,7 +217,7 @@ class bufferServer(Elem):
 
     def _receive(self, message):
         if message[1]:
-            self.lines.append(message[2])
+            self.lines.append(message[2:].decode("utf-8"))
             if len(self.lines) > 100:
                 self.lines = self.lines[-100:]
         else:
@@ -242,18 +253,21 @@ class lineServer(Elem):
         self.background = background
         self.color = color
         self.dataArray = []
-        self.variables = {}
+        self.variables = []
+        self.variableNames = {}
         for v in filter(lambda x: isinstance(x, _Val), self.pattern):
             if isinstance(v, Vstr):
-                self.variables[v.name] = ""
+                self.variableNames[v.name] = len(self.variableNames)
+                self.variables.append("")
             elif isinstance(v, Vint):
-                self.variables[v.name] = 0
+                self.variableNames[v.name] = len(self.variableNames)
+                self.variables.append(0)
 
     def _receive(self, data):
         if data[2]:
-            self.variables[data[1]] += data[3]
+            self.variables[data[1]] += int.from_bytes(data[3:], byteorder='little')
         else:
-            self.variables[data[1]] = data[3]
+            self.variables[data[1]] = data[3:].decode("utf-8")
 
     def _getDataChunk(self, i):
         if len(self.dataArray) > i:
@@ -268,9 +282,9 @@ class lineServer(Elem):
     def _renderVar(self, x: _Val):
         buf = []
         if isinstance(x, Vstr):
-            buf.append(self.variables[x.name])
+            buf.append(self.variables[self.variableNames[x.name]])
         elif isinstance(x, Vint):
-            buf.append(str(self.variables[x.name]))
+            buf.append(str(self.variables[self.variableNames[x.name]]))
         if x.width is not None:
             return fillSpaces(''.join(buf), x.width, x.align)
         else:
